@@ -1,22 +1,8 @@
-/***************************************************************************
- *   This file is part of KDevelop                                         *
- *   Copyright 2008 Niko Sams <niko.sams@gmail.com>                        *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
- *   published by the Free Software Foundation; either version 2 of the    *
- *   License, or (at your option) any later version.                       *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
- *   License along with this program; if not, write to the                 *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2008 Niko Sams <niko.sams@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "declarationbuilder.h"
 
@@ -1113,8 +1099,8 @@ void DeclarationBuilder::declareVariable(DUContext* parentCtx, AbstractType::Ptr
                     }
                     if ( (*it)->abstractType() && !(*it)->abstractType()->equals(type.data()) ) {
                         // if it's currently mixed and we now get something more definite, use that instead
-                        if ( ReferenceType::Ptr rType = ReferenceType::Ptr::dynamicCast((*it)->abstractType()) ) {
-                            if ( IntegralType::Ptr integral = IntegralType::Ptr::dynamicCast(rType->baseType()) ) {
+                        if ( auto rType = (*it)->abstractType().dynamicCast<ReferenceType>() ) {
+                            if ( auto integral = rType->baseType().dynamicCast<IntegralType>() ) {
                                 if ( integral->dataType() == IntegralType::TypeMixed ) {
                                     // referenced mixed to referenced @p type
                                     ReferenceType::Ptr newType(new ReferenceType());
@@ -1124,7 +1110,7 @@ void DeclarationBuilder::declareVariable(DUContext* parentCtx, AbstractType::Ptr
                                 }
                             }
                         }
-                        if ( IntegralType::Ptr integral = IntegralType::Ptr::dynamicCast((*it)->abstractType()) ) {
+                        if ( auto integral = (*it)->abstractType().dynamicCast<IntegralType>() ) {
                             if ( integral->dataType() == IntegralType::TypeMixed ) {
                                 // mixed to @p type
                                 (*it)->setType(type);
@@ -1132,11 +1118,11 @@ void DeclarationBuilder::declareVariable(DUContext* parentCtx, AbstractType::Ptr
                             }
                         }
                         // else make it unsure
-                        UnsureType::Ptr unsure = UnsureType::Ptr::dynamicCast((*it)->abstractType());
+                        auto unsure = (*it)->abstractType().dynamicCast<UnsureType>();
                         // maybe it's referenced?
-                        ReferenceType::Ptr rType = ReferenceType::Ptr::dynamicCast((*it)->abstractType());
+                        auto rType = (*it)->abstractType().dynamicCast<ReferenceType>();
                         if ( !unsure && rType ) {
-                            unsure = UnsureType::Ptr::dynamicCast(rType->baseType());
+                            unsure = rType->baseType().dynamicCast<UnsureType>();
                         }
                         if ( !unsure ) {
                             unsure = UnsureType::Ptr(new UnsureType());
@@ -1422,7 +1408,7 @@ void DeclarationBuilder::visitFunctionCallParameterListElement(FunctionCallParam
     if ( m_findVariable.node && m_currentFunctionType &&
             m_currentFunctionType->arguments().count() > m_functionCallParameterPos) {
         ReferenceType::Ptr refType = m_currentFunctionType->arguments()
-                                        .at(m_functionCallParameterPos).cast<ReferenceType>();
+                                        .at(m_functionCallParameterPos).dynamicCast<ReferenceType>();
         if ( refType ) {
             // this argument is referenced, so if the node contains undeclared variables we have
             // to declare them with a NULL type, see also:
@@ -1642,7 +1628,16 @@ void DeclarationBuilder::visitUseStatement(UseStatementAst* node)
     DeclarationBuilderBase::visitUseStatement(node);
 }
 
-void DeclarationBuilder::visitUseNamespace(UseNamespaceAst* node)
+void DeclarationBuilder::visitUseNamespaceOrUseGroupedNamespace(UseNamespaceOrUseGroupedNamespaceAst* node)
+{
+    if (node->compoundNamespace) {
+        // TODO
+    } else {
+        visitNonGroupedUseNamespace(node);
+    }
+}
+
+void DeclarationBuilder::visitNonGroupedUseNamespace(UseNamespaceOrUseGroupedNamespaceAst* node)
 {
     DUChainWriteLocker lock;
     bool isConstIdentifier = ( m_useNamespaceType == ConstantDeclarationType );
